//
//  Kiban.swift
//  Kiban2
//
//  Created by Kaz Yoshikawa on 2015/04/30.
//  Copyright (c) 2015年 Electricwoods LLC. All rights reserved.
//

import Foundation

/*
extension String {

	func lines() -> [String] {
		var lines = [String]()
		self.enumerateLines { (line, stop) -> () in
			lines.append(line)
		}
		return lines
	}

}
*/

class KibanScanner : NSObject {

	var scanner: NSScanner

	init(string: String) {
		scanner = NSScanner(string: string)
		scanner.charactersToBeSkipped = NSCharacterSet.whitespaceCharacterSet()
	}
	
	func scan先手後手() -> Player? {
		for (key, value) in Player.table {
			var string: NSString? = nil
			if scanner.scanString(key, intoString: &string) {
				return value
			}
		}
		return nil
	}

	func scan持駒() -> Captured? {
		var captured = Captured()
		let 先手後手 = self.scan先手後手()
		scanner.scanString("持駒:", intoString: nil)
		if !scanner.scanString("なし", intoString: nil) {
			var table: [String: Piece] = ["飛": .飛, "角": .角, "金": .金, "銀": .銀, "桂": .桂, "香": .香, "歩": .歩]
			for (key, value) in Piece.table {
				var string: NSString? = nil
				var count: Int = 1
				if scanner.scanString(key, intoString: &string) {
					scanner.scanInteger(&count)
					captured[value] = count
					scanner.scanString(",", intoString: nil)
				}
			}
		}
		return captured
	}

	func scan駒面() -> Face? {
		for (key, value) in Face.table {
			if scanner.scanString(key, intoString: nil) {
				return value
			}
		}
		return nil
	}

	func scanSquare() -> Square? {
		var string: NSString? = nil
		if let 先手後手 = scan先手後手(), 駒面 = scan駒面() {
			return Square(先後: 先手後手, 駒面: 駒面)
		}
		/* - following code somehow not working "　" zenkaku-space
		else if scanner.scanCharactersFromSet(NSCharacterSet(charactersInString: "　"), intoString: nil) {
			return Square()
		} */
		else {
			return Square()
		}
	}
	
}

let numberOfSquares = 81
let numberOfColums = 9
let numberOfRows = 9

enum Column : Int8, Printable { // right <- left
	case １, ２, ３, ４, ５, ６, ７, ８, ９

	static let table: [String: Column] = [
		"１": .１, "２": .２, "３": .３, "４": .４, "５": .５, "６": .６, "７": .７, "８": .８, "９": .９
	]

	init?(string: String) {
		for (key, value) in Column.table {
			if string == key {
				self = value
				return
			}
		}
		return nil
	}
	
	var description: String {
		for (key, value) in Column.table {
			if value == self {
				return key
			}
		}
		return "?"
	}
}

enum Row : Int8, Printable { // top -> bottom
	case 一, 二, 三, 四, 五, 六, 七, 八, 九

	private  static let table: [String: Row] = [
		"一": .一, "二": .二, "三": .三, "四": .四, "五": .五, "六": .六, "七": .七, "八": .八, "九": .九
	]

	init?(string: String) {
		for (key, value) in Row.table {
			if string == key {
				self = value
				return
			}
		}
		return nil
	}

	var description: String {
		for (key, value) in Row.table {
			if value == self {
				return key
			}
		}
		return "?"
	}

	var territory: Territory {
		return Territory(row: self)
	}
}

// MARK: Territory

enum Territory : Int8 {
	case 先手, 中立, 後手

	init(row: Row) {
		switch row {
		case .一, .二, .三: self = .後手
		case .四, .五, .六: self = .中立
		case .七, .八, .九: self = .先手
		}
	}

	init(先後: Player) {
		switch (先後) {
		case .先手: self = .先手
		case .後手: self = .後手
		}
	}
}


// MARK: Position

enum Position : Int8, Equatable, Hashable, Printable { // Hashable

	case １一, １二, １三, １四, １五, １六, １七, １八, １九
	case ２一, ２二, ２三, ２四, ２五, ２六, ２七, ２八, ２九
	case ３一, ３二, ３三, ３四, ３五, ３六, ３七, ３八, ３九

	case ４一, ４二, ４三, ４四, ４五, ４六, ４七, ４八, ４九
	case ５一, ５二, ５三, ５四, ５五, ５六, ５七, ５八, ５九
	case ６一, ６二, ６三, ６四, ６五, ６六, ６七, ６八, ６九

	case ７一, ７二, ７三, ７四, ７五, ７六, ７七, ７八, ７九
	case ８一, ８二, ８三, ８四, ８五, ８六, ８七, ８八, ８九
	case ９一, ９二, ９三, ９四, ９五, ９六, ９七, ９八, ９九


	static let min: Int8 = Position.１一.rawValue
	static let max: Int8 = Position.９九.rawValue + 1
	

	init(column: Column, row: Row) {
		self = Position(rawValue: column.rawValue * Int8(numberOfRows) + row.rawValue)!
	}

	init?(columnIndex: Int, rowIndex: Int) {
		if 0...8 ~= columnIndex && 0...8 ~= rowIndex {
			self = Position(rawValue: Int8(columnIndex * numberOfRows + rowIndex))!
		}
		else {
			return nil
		}
	}

	init?(string: String) {
		if let position = Position.table[string] {
			self = position
		}
		else {
			return nil
		}
	}
	
	private static let table: [String: Position] = [
		"１一": .１一, "１二": .１二, "１三": .１三, "１四": .１四, "１五": .１五, "１六": .１六, "１七": .１七, "１八": .１八, "１九": .１九,
		"２一": .２一, "２二": .２二, "２三": .２三, "２四": .２四, "２五": .２五, "２六": .２六, "２七": .２七, "２八": .２八, "２九": .２九,
		"３一": .３一, "３二": .３二, "３三": .３三, "３四": .３四, "３五": .３五, "３六": .３六, "３七": .３七, "３八": .３八, "３九": .３九,
		"４一": .４一, "４二": .４二, "４三": .４三, "４四": .４四, "４五": .４五, "４六": .４六, "４七": .４七, "４八": .４八, "４九": .４九,
		"５一": .５一, "５二": .５二, "５三": .５三, "５四": .５四, "５五": .５五, "５六": .５六, "５七": .５七, "５八": .５八, "５九": .５九,
		"６一": .６一, "６二": .６二, "６三": .６三, "６四": .６四, "６五": .６五, "６六": .６六, "６七": .６七, "６八": .６八, "６九": .６九,
		"７一": .７一, "７二": .７二, "７三": .７三, "７四": .７四, "７五": .７五, "７六": .７六, "７七": .７七, "７八": .７八, "７九": .７九,
		"８一": .８一, "８二": .８二, "８三": .８三, "８四": .８四, "８五": .８五, "８六": .８六, "８七": .８七, "８八": .８八, "８九": .８九,
		"９一": .９一, "９二": .９二, "９三": .９三, "９四": .９四, "９五": .９五, "９六": .９六, "９七": .９七, "９八": .９八, "９九": .９九
	]

	static let positions: [Position] = [ // 段は右から左へ
		.１一, .１二, .１三, .１四, .１五, .１六, .１七, .１八, .１九,
		.２一, .２二, .２三, .２四, .２五, .２六, .２七, .２八, .２九,
		.３一, .３二, .３三, .３四, .３五, .３六, .３七, .３八, .３九,
		.４一, .４二, .４三, .４四, .４五, .４六, .４七, .４八, .４九,
		.５一, .５二, .５三, .５四, .５五, .５六, .５七, .５八, .５九,
		.６一, .６二, .６三, .６四, .６五, .６六, .６七, .６八, .６九,
		.７一, .７二, .７三, .７四, .７五, .７六, .７七, .７八, .７九,
		.８一, .８二, .８三, .８四, .８五, .８六, .８七, .８八, .８九,
		.９一, .９二, .９三, .９四, .９五, .９六, .９七, .９八, .９九
	]

	var columIndex: Int8 {
		return self.rawValue / Int8(numberOfRows)
	}

	var rowIndex: Int8 {
		return self.rawValue % Int8(numberOfRows)
	}

	var 列: Column {
		return Column(rawValue: self.columIndex)!
	}

	var 段: Row {
		return Row(rawValue: self.rowIndex)!
	}
	
	func offsetted(x: Int, _ y: Int) -> Position? {
		return Position(columnIndex: x, rowIndex: y)
	}

	var description: String {
		for (key, value) in Position.table {
			if value == self {
				return key
			}
		}
		return "huh?"
	}

	var 陣 : Territory {
		return self.段.territory
	}
}


// MARK: Piece

enum Piece : Int8, Printable {
	case 歩, 香, 桂, 銀, 金, 角, 飛, 王

	private static let table: [String: Piece] = [
		"王": .王, "飛": .飛, "角": .角, "金": .金, "銀": .銀, "桂": .桂, "香": .香, "歩": .歩,
	]

	init?(string: String) {
		if let piece = Piece.table[string] {
			self = piece
		}
		else {
			return nil
		}
	}

	var description: String {
		for (key, value) in Piece.table {
			if value == self {
				return key
			}
		}
		return "huh?"
	}
	
	var 駒面: Face {
		switch self {
		case 歩: return .歩
		case 香: return .香
		case 桂: return .桂
		case 銀: return .銀
		case 金: return .金
		case 角: return .角
		case 飛: return .飛
		case 王: return .王
		}
	}

	var promotable: Bool {
		switch self {
		case 歩, 香, 桂, 銀, 角, 飛: return true
		case 金, 王: return false
		}
	}

	var 駒数: UInt8 {
		switch self {
		case 歩: return 18
		case 香, 桂, 銀, 金: return 4
		case 角, 飛: return 2
		case 王: return 2
		}
	}
}


// MARK: Face

enum Face : Int8, Printable {
	case 歩, 香, 桂, 銀, 金, 角, 飛, 王
	case と, 杏, 圭, 全, 馬, 竜

	static let table: [String: Face] = [
		"歩": .歩,
		"香": .香,
		"桂": .桂,
		"銀": .銀,
		"金": .金,
		"角": .角,
		"飛": .飛,
		"王": .王, "玉": .王,
		"と": .と,
		"杏": .杏,
		"圭": .圭,
		"全": .全,
		"馬": .馬,
		"竜": .竜, "龍": .竜
	]

	init?(string: String) {
		if let 駒面 = Face.table[string] {
			self = 駒面
		}
		else {
			return nil
		}
	}
	
	var 駒: Piece {
		switch self {
		case 歩, と: return .歩
		case 香, 杏: return .香
		case 桂, 圭: return .桂
		case 銀, 全: return .銀
		case 金: return .金
		case 角, 馬: return .角
		case 飛, 竜: return .飛
		case 王: return .王
		}
	}

	var string: String {
		for (key, value) in Face.table {
			if self == value {
				return key
			}
		}
		return ""
	}

	var description: String {
		return self.string
	}

	func 移動可能なオフセット(#駒面: Face, 先後: Player) -> [(Int, Int)] {
		let y = 先後.direction
		assert(abs(y) == 1)

		switch 駒面 {
		case 歩: return [(0, y)]
		case 香: return []
		case 桂: return [(-1, y * 2), (1, y * 2)]
		case 銀: return [(-1, y), (0, y), (1, y),  (-1, -y), (1, -y)]
		case 角: return []
		case 飛: return []
		case 王, 馬, 竜: return [(-1, -1), (0, -1), (1, -1),  (-1, 0), (1, 0),  (-1, 1), (0, 1), (1, 1)]
		case 金, と, 杏, 圭, 全: return [(-1, -1), (0, -1), (1, -1),  (-1, 0), (1, 0),  (0, 1)]
		}
	}

	func 移動可能なベクトル(#駒面: Face, 先後: Player) -> [(Int, Int)] {
		let y = 先後.direction
		assert(abs(y) == 1)

		switch 駒面 {
		case 香: return [(0, y)]
		case 角, 馬: return [(y, y), (-y, y), (y, -y), (-y, -y)]
		case 飛, 竜: return [(0, y), (0, -y), (y, 0), (-y, 0)]
		default: return []
		}
	}

	func 成る() -> Face {
		switch self {
		case 歩: return と
		case 香: return 杏
		case 桂: return 圭
		case 銀: return 全
		case 角: return 馬
		case 飛: return 竜
		case 金, 王: fatalError("should have checked")
		case と, 杏, 圭, 全, 馬, 竜: return self
		}
	}
	
	var 成る事は可能か: Bool {
		switch self {
		case 歩, 香, 桂, 銀, 角, 飛: return true
		case 金, 王, と, 杏, 圭, 全, 馬, 竜: return false
		}
	}

	var 成駒か: Bool {
		switch self {
		case と, 杏, 圭, 全, 馬, 竜: return true
		default: return false
		}
	}

	
}


// MARK: Captured

struct Captured: Printable {
	// can be dictionary but this saves some memoery foot print
	var 歩, 香, 桂, 銀, 金, 角, 飛: Int8
	
	init(歩: Int8, 香: Int8, 桂: Int8, 銀: Int8, 金: Int8, 角: Int8, 飛: Int8) {
		self.歩 = 歩 ; self.香 = 香 ; self.桂 = 桂 ;
		self.銀 = 銀 ; self.金 = 金
		self.角 = 角 ; self.飛 = 飛
	}

	init(歩: Int, 香: Int, 桂: Int, 銀: Int, 金: Int, 角: Int, 飛: Int) {
		self.歩 = Int8(歩) ; self.香 = Int8(香); self.桂 = Int8(桂)
		self.銀 = Int8(銀) ; self.金 = Int8(金)
		self.角 = Int8(角) ; self.飛 = Int8(飛)
	}

	init() {
		self.init(歩: 0, 香: 0, 桂: 0, 銀: 0, 金: 0, 角: 0, 飛: 0)
	}

	init(string: String) {
		self.init()

		// "飛角金2桂歩4", "角,銀3,香,歩", "なし"
		let scanner = KibanScanner(string: string)
		scanner.scan先手後手()
		scanner.scanner.scanString("持駒:", intoString: nil)
		for (key, value) in Captured.table {
			var string: NSString? = nil
			var count: Int = 1
			if scanner.scanner.scanString(key, intoString: &string) {
				scanner.scanner.scanInteger(&count)
				self[value] = count
				scanner.scanner.scanString(",", intoString: nil)
			}
		}
		println("\(self.dictionaryRepresentation)")
	}

	static let table: [String: Piece] = [
		"飛": .飛, "角": .角, "金": .金, "銀": .銀, "桂": .桂, "香": .香, "歩": .歩
	]

	var dictionaryRepresentation: [String: Int8] {
		return [
			"飛": self.飛, "角": self.角, "金": self.金, "銀": self.銀, "桂": self.桂, "香": self.香, "歩": self.歩
		]
	}

	var string: String {

		func symbolWithCount(symbol: String, count: Int8) -> String? {
			if count > 0 {
				var string = ""
				string += symbol
				if count > 1 {
					string += "\(count)"
				}
				return string
			}
			return nil
		}

		var strings = [String]()
		for (key, value) in self.dictionaryRepresentation {
			if let string = symbolWithCount(key, value) {
				strings.append(string)
			}
		}

		var string = "持駒:"
		if strings.count > 0 {
			string += (strings as NSArray).componentsJoinedByString(",")
		}
		else {
			string += "なし"
		}
		return string
	}

	func string(#player: Player) -> String {
		return player.string + self.string
	}

	subscript (key: Piece) -> Int {
		get {
			switch key {
			case .歩: return Int(歩)
			case .香: return Int(香)
			case .桂: return Int(桂)
			case .銀: return Int(銀)
			case .金: return Int(金)
			case .角: return Int(角)
			case .飛: return Int(飛)
			default: return 0
			}
		}
		set {
			switch key {
			case .歩: 歩 = Int8(newValue)
			case .香: 香 = Int8(newValue)
			case .桂: 桂 = Int8(newValue)
			case .銀: 銀 = Int8(newValue)
			case .金: 金 = Int8(newValue)
			case .角: 角 = Int8(newValue)
			case .飛: 飛 = Int8(newValue)
			default: break
			}
		}
	}

	var description: String {
		return self.string
	}

	mutating func increment(#piece: Piece) {
		switch (piece) {
		case .歩: self.歩++; assert(self.歩 <= 18)
		case .香: self.香++; assert(self.香 <= 4)
		case .桂: self.桂++; assert(self.桂 <= 4)
		case .銀: self.銀++; assert(self.銀 <= 4)
		case .金: self.金++; assert(self.金 <= 4)
		case .角: self.角++; assert(self.角 <= 2)
		case .飛: self.飛++; assert(self.飛 <= 2)
		case .王: break /* aah! nice try! */
		}
	}

	mutating func decrement(#piece: Piece) {
		switch (piece) {
		case .歩: self.歩--; assert(self.歩 >= 0)
		case .香: self.香--; assert(self.香 >= 0)
		case .桂: self.桂--; assert(self.桂 >= 0)
		case .銀: self.銀--; assert(self.銀 >= 0)
		case .金: self.金--; assert(self.金 >= 0)
		case .角: self.角--; assert(self.角 >= 0)
		case .飛: self.飛--; assert(self.飛 >= 0)
		case .王: break /* aah! nice try! */
		}
	}

	var integerValue: UInt32 {
		var value: UInt64 = 0
		value += UInt64(飛)
		value *= UInt64(Piece.角.駒数+1) ; value += UInt64(角)
		value *= UInt64(Piece.金.駒数+1) ; value += UInt64(金)
		value *= UInt64(Piece.銀.駒数+1) ; value += UInt64(銀)
		value *= UInt64(Piece.桂.駒数+1) ; value += UInt64(桂)
		value *= UInt64(Piece.香.駒数+1) ; value += UInt64(香)
		value *= UInt64(Piece.歩.駒数+1) ; value += UInt64(歩)
		assert(value <= 0x1_a17b)
		return UInt32(value)
	}

	init(integerValue: UInt32) {
		var value = UInt64(integerValue)
		歩 = Int8(value % UInt64(Piece.歩.駒数+1)) ; value /= UInt64(Piece.歩.駒数+1)
		香 = Int8(value % UInt64(Piece.香.駒数+1)) ; value /= UInt64(Piece.香.駒数+1)
		桂 = Int8(value % UInt64(Piece.桂.駒数+1)) ; value /= UInt64(Piece.桂.駒数+1)
		銀 = Int8(value % UInt64(Piece.銀.駒数+1)) ; value /= UInt64(Piece.銀.駒数+1)
		金 = Int8(value % UInt64(Piece.金.駒数+1)) ; value /= UInt64(Piece.金.駒数+1)
		角 = Int8(value % UInt64(Piece.角.駒数+1)) ; value /= UInt64(Piece.角.駒数+1)
		飛 = Int8(value % UInt64(Piece.飛.駒数+1))
	}
}


// MARK: Player

enum Player : Int8, Printable {
	case 先手, 後手

	private static let table: [String: Player] = [
		"☗": .先手, "▲": .先手, "先手": .先手,
		"☖": .後手, "△": .後手, "後手": .後手
	]

	init?(string: String) {
		if let player = Player.table[string] {
			self = player
		}
		else {
			return nil
		}
	}

	var string: String {
		switch self {
		case .先手: return "☗"
		case .後手: return "☖"
		}
	}
	
	var direction: Int {
		switch self {
		case .先手: return -1
		case .後手: return 1
		}
	}

	var description: String {
		return self.string
	}

	var 相手: Player {
		switch self {
		case .先手: return .後手
		case .後手: return .先手
		}
	}
	
	func square(駒面: Face) -> Square {
		return Square(先後: self, 駒面: 駒面)
	}
	
}


// MARK: Square

enum Square : Int8, Printable {
	case 空

	case 先歩, 先香, 先桂, 先銀, 先金, 先角, 先飛, 先玉
	case 先と, 先杏, 先圭, 先全, 先馬, 先竜

	case 後歩, 後香, 後桂, 後銀, 後金, 後角, 後飛, 後王
	case 後と, 後杏, 後圭, 後全, 後馬, 後竜

	static let max = 29

	init(先後: Player, 駒面: Face) {
		switch (先後, 駒面) {
		case (.先手, .歩): self = .先歩
		case (.先手, .香): self = .先香
		case (.先手, .桂): self = .先桂
		case (.先手, .銀): self = .先銀
		case (.先手, .金): self = .先金
		case (.先手, .角): self = .先角
		case (.先手, .飛): self = .先飛
		case (.先手, .王): self = .先玉
		case (.先手, .と): self = .先と
		case (.先手, .杏): self = .先杏
		case (.先手, .圭): self = .先圭
		case (.先手, .全): self = .先全
		case (.先手, .馬): self = .先馬
		case (.先手, .竜): self = .先竜

		case (.後手, .歩): self = .後歩
		case (.後手, .香): self = .後香
		case (.後手, .桂): self = .後桂
		case (.後手, .銀): self = .後銀
		case (.後手, .金): self = .後金
		case (.後手, .角): self = .後角
		case (.後手, .飛): self = .後飛
		case (.後手, .王): self = .後王
		case (.後手, .と): self = .後と
		case (.後手, .杏): self = .後杏
		case (.後手, .圭): self = .後圭
		case (.後手, .全): self = .後全
		case (.後手, .馬): self = .後馬
		case (.後手, .竜): self = .後竜
		}
	}

	init() {
		self = .空
	}

	var 先後: Player? {
		switch self {
		case 空: return nil
		case 先歩, 先香, 先桂, 先銀, 先金, 先角, 先飛, 先玉, 先と, 先杏, 先圭, 先全, 先馬, 先竜: return .先手
		case 後歩, 後香, 後桂, 後銀, 後金, 後角, 後飛, 後王, 後と, 後杏, 後圭, 後全, 後馬, 後竜: return .後手
		}
	}

	var 駒面: Face? {
		switch self {
		case 先歩, 後歩: return .歩
		case 先香, 後香: return .香
		case 先桂, 後桂: return .桂
		case 先銀, 後銀: return .銀
		case 先金, 後金: return .金
		case 先角, 後角: return .角
		case 先飛, 後飛: return .飛
		case 先玉, 後王: return .王

		case 先と, 後と: return .と
		case 先杏, 後杏: return .杏
		case 先圭, 後金: return .圭
		case 先全, 後角: return .全
		case 先馬, 後馬: return .馬
		case 先竜, 後竜: return .竜
		default: return nil
		}
	}

//		"☗": .先手, "▲": .先手, "先手": .先手,
//		"☖": .後手, "△": .後手, "後手": .後手

	private static let table : [String: Square] = [
		"　　": .空,
		"☗歩": .先歩, "☗香": .先香, "☗桂": .先桂, "☗銀": .先銀, "☗金": .先金, "☗角": .先角, "☗飛": .先飛, "☗玉": .先玉,
		"☗と": .先と, "☗杏": .先杏, "☗圭": .先圭, "☗全": .先全, "☗馬": .先馬, "☗竜": .先竜,
		"☖歩": .後歩, "☖香": .後香, "☖桂": .後桂, "☖銀": .後銀, "☖金": .後金, "☖角": .後角, "☖飛": .後飛, "☖王": .後王,
		"☖と": .後と, "☖杏": .後杏, "☖圭": .後圭, "☖全": .後全, "☖馬": .後馬, "☖竜": .後竜,
		"☗王": .先玉, "☗龍": .先竜, "☖龍": .後竜,
	]
	
	var string: String {
		for (key, value) in Square.table {
			if self == value {
				return key
			}
		}
		return "?"
	}

	var description: String {
		for (key, value) in Square.table {
			if value == self {
				return key
			}
		}
		return "?"
	}
}

enum Handicap {
	case 平手 //, 香落ち, 角落ち, 飛車落ち, 飛香落ち, 二枚落ち, 四枚落ち, 六枚落ち


	static let 平手初期盤面: String =
			"☖持駒:なし\r" +
			"|☖香|☖桂|☖銀|☖金|☖王|☖金|☖銀|☖桂|☖香|\r" +
			"|　　|☖飛|　　|　　|　　|　　|　　|☖角|　　|\r" +
			"|☖歩|☖歩|☖歩|☖歩|☖歩|☖歩|☖歩|☖歩|☖歩|\r" +
			"|　　|　　|　　|　　|　　|　　|　　|　　|　　|\r" +
			"|　　|　　|　　|　　|　　|　　|　　|　　|　　|\r" +
			"|　　|　　|　　|　　|　　|　　|　　|　　|　　|\r" +
			"|☗歩|☗歩|☗歩|☗歩|☗歩|☗歩|☗歩|☗歩|☗歩|\r" +
			"|　　|☗角|　　|　　|　　|　　|　　|☗飛|　　|\r" +
			"|☗香|☗桂|☗銀|☗金|☗王|☗金|☗銀|☗桂|☗香|\r" +
			"☗持駒:なし\r"
}

/*
enum MoveKind {
	case 動, 投了, 中断, 千日手, 時間切れ, 反則負け, 持将棋
}
*/


// MARK: Action

enum Action : Printable {
	case 動(先後: Player, 移動前の位置: Position, 移動後の位置: Position, 移動後の駒面: Face)
	case 打(先後: Player, 位置:Position, 駒:Piece)
	case 投了(先後: Player)

	var description: String {
		var string = ""
		switch self {
		case .動(let 先後, let 移動前の位置, let 移動後の位置, let 移動後の駒面):
			string += "\(先後)\(移動前の位置)->\(移動後の位置)\(移動後の駒面)"
		
		default: break
		}
		return string
	}
}

/*
struct Move : Equatable, Hashable {
	var 先後: Player
	var 動作: MoveKind
	var 移動前: Position
	var 移動後: Position
	var 駒: Face
	var hashValue: Int {
		return 先後.hashValue + 動作.hashValue + 移動前.hashValue + 移動後.hashValue
	}
}
*/


// MARK: Board

class Board : Equatable, Printable, SequenceType {

	var score: Int = 0
	weak var previous: Board?

	var squares: [Square]
	var captured: [Player: Captured]
	var turn: Player

	init() {
		squares = [Square](count: numberOfSquares, repeatedValue: Square.空)
		captured = [.先手: Captured(), .後手: Captured()]
		turn = .先手
	}

	init(turn: Player, squares: [Square], captured: [Player: Captured]) {
		self.squares = squares
		self.captured = captured
		self.turn = turn
	}

	init?(string: String) {
		squares = [Square](count: numberOfSquares, repeatedValue: Square.空)
		captured = [.先手: Captured(), .後手: Captured()]
		turn = .先手

		var lines = string.lines()
		if lines.count >= (9 + 2) {
			captured[.後手] = Captured(string: lines[0])
			for rowIndex in 0...8 {
				let line = lines[rowIndex + 1]
				println("\(line)")
				var scanner = KibanScanner(string: line)
				scanner.scanner.scanString("|", intoString: nil)
				for columnIndex in reverse(0...8) {
					if let square = scanner.scanSquare() {
						self[columnIndex, rowIndex] = square
						scanner.scanner.scanString("|", intoString: nil)
					}
				}
			}
			captured[.先手] = Captured(string: lines[10])
		}
		else {
			return nil
		}
	}

	init(board: Board) {
		self.squares = board.squares
		self.captured = board.captured
		self.turn = board.turn.相手
	}

	class func squareIndexFrom(column columnIndex: Int, row rowIndex: Int) -> Int? {
		if 0...8 ~= columnIndex && 0...8 ~= rowIndex {
			return columnIndex * numberOfRows + rowIndex
		}
		return nil
	}

	subscript(columnIndex: Int, rowIndex: Int) -> Square? {
		get {
			if let index = Board.squareIndexFrom(column: columnIndex, row: rowIndex) {
				return squares[index]
			}
			return nil
		}
		set {
			if let index = Board.squareIndexFrom(column: columnIndex, row: rowIndex), newValue = newValue {
				squares[index] = newValue
			}
		}
	}
	
	subscript(position: Position) -> Square {
		get {
			return squares[Int(position.rawValue)]
		}
		set {
			squares[Int(position.rawValue)] = newValue
		}
	}
	
	var description: String {
		var string = String()
		if let captured = captured[.後手] {
			string += captured.string(player: .後手) + "\r"
		}
		for rowIndex in 0...8 {
			string += "|"
			for columnIndex in reverse(0...8) {
				if let square = self[columnIndex, rowIndex] {
					string += square.string
					string += "|"
				}
			}
			string += "\r"
		}
		if let captured = captured[.先手] {
			string += captured.string(player: .先手) + "\r"
		}
		return string
	}

	func 駒の移動可能位置(col: Int, _ row: Int) -> [Position] {
		var positions = [Position]()
		if let square = self[col, row], 先後 = square.先後, 駒面 = square.駒面 {

			// find movable positions for single step like 歩, 桂, 金, ...
			for (dx, dy) in 駒面.移動可能なオフセット(駒面: 駒面, 先後: 先後) {
				let (x, y) = (col + dx, row + dy)
				if let dsquare = self[x, y] {
					if let dplayer = dsquare.先後 where dplayer == 先後 {
					}
					else if let position = Position(columnIndex: x, rowIndex: y) {
						positions.append(position)
					}
				}
			}

			// find movable positions for distant move like 香, 飛, 角
			for (vx, vy) in 駒面.移動可能なベクトル(駒面: 駒面, 先後: 先後) {
				for var xy = (col + vx, row + vy) ; (0...8, 0...8) ~= xy ; xy += (vx, vy) {
					let (x, y) = xy
					if let dsquare = self[x, y] {
						if let dplayer = dsquare.先後 { // ran into yours
							if dplayer == 先後 {
								break // leave inner loop
							}
							else { // ran into opponent
								if let position = Position(columnIndex: x, rowIndex: y) {
									positions.append(position)
									break // leave inner loop
								}
							}
						}
						else if let position = Position(columnIndex: x, rowIndex: y) {
							positions.append(position)
						}
					}
					else {
						break // out of the board
					}
				}
			}
		}
		return positions
	}

	func 駒の移動可能位置(position: Position) -> [Position] {
		return 駒の移動可能位置(Int(position.columIndex), Int(position.rowIndex))
	}

	// in order to avoid huge number of linear search, build a lookup dictionary from square to position
	func 駒からマスへの辞書() -> [Square: [Position]] {
		var dictionary = [Square: [Position]]()
		for (index, square) in enumerate(squares) {
			if let position = Position(rawValue: Int8(index)) {
				var positions = dictionary[square] ?? []
				positions.append(position)
				dictionary[square] = positions
			}
		}
		return dictionary
	}

	func forward(action: Action) -> Board {
		var board = Board(board: self)
		board.previous = self
		switch action {
		case .動(let 先手後手, let 移動前の位置, let 移動後の位置, let 移動後の駒面):
			let 移動前のマス = self[移動前の位置]
			let 移動後のマス = self[移動後の位置]
			if let 移動前の駒面 = 移動前のマス.駒面 where 移動前の駒面.駒 == 移動後の駒面.駒 {
				if let 移動後の駒面 = 移動後のマス.駒面, 移動後のマスの駒の先手後手 = 移動後のマス.先後 {
					// 相手の駒を取る
					assert(移動後のマスの駒の先手後手 == 先手後手.相手)
					board.captured[先手後手]!.increment(piece: 移動後の駒面.駒)
				}
				board[移動前の位置] = .空
				board[移動後の位置] = Square(先後: 先手後手, 駒面: 移動後の駒面)
			}
			else { /*論理エラー*/ }
			break
		default:
			break
		}
		return board
	}


	func processPositions(f: (Position) -> ()) {
		for col in 0...8 {
			for row in 0...8 {
				if let position = Position(columnIndex: col, rowIndex: row) {
					f(position)
				}
			}
		}
	}

	func 全ての可能指手() -> [Action] {
		var actions = [Action]()
		for position in self.positions() {
			let square = self[position]
			if let 先後 = square.先後, 駒面 = square.駒面 where 先後 == turn {
				for destination in 駒の移動可能位置(position) {
					if 不成で移動可能か(先後: 先後, 移動前の位置: position, 移動後の位置: destination, 移動前の駒面: 駒面) {
						actions.append(Action.動(先後: 先後, 移動前の位置: position, 移動後の位置: destination, 移動後の駒面: 駒面))
					}
					if 成る事ができるか(先後: 先後, 移動前の位置: position, 移動後の位置: destination, 移動前の駒面: 駒面) {
						actions.append(Action.動(先後: 先後, 移動前の位置: position, 移動後の位置: destination, 移動後の駒面: 駒面.成る()))
					}
				}
			}
		}
		return actions
	}
	
	func 成る事ができるか(#先後: Player, 移動前の位置: Position, 移動後の位置: Position, 移動前の駒面: Face) -> Bool {
		let opponent = 先後.相手
		let 敵陣 = Territory(先後: opponent)
		let 移動前 = 移動前の位置.陣
		let 移動後 = 移動後の位置.陣
		
		return (移動前 == 敵陣 || 移動後 == 敵陣) && 移動前の駒面.成る事は可能か && !移動前の駒面.成駒か
	}

	func 不成で移動可能か(#先後: Player, 移動前の位置: Position, 移動後の位置: Position, 移動前の駒面: Face) -> Bool {

		switch (移動後の位置.段, 移動前の駒面, 先後) {
		case (.一, .歩, .先手): return false
		case (.一, .香, .先手): return false
		case (.一, .桂, .先手): return false
		case (.二, .桂, .先手): return false

		case (.八, .桂, .先手): return false
		case (.九, .桂, .後手): return false
		case (.九, .香, .後手): return false
		case (.九, .歩, .後手): return false

		default: return true // 成駒の移動は不成で移動とみなす
		}
	}

	func 指定位置へ移動可能な全ての駒の位置(target: Position) -> [Position] {
		// （動作未確認）
		var positions = [Position]()
		// 盤上の全ての駒について調べる
		for position in self {
			// そこに駒があれば、その駒の全ての移動可能先を調べる
			for destination in 駒の移動可能位置(position) {
				if destination == target {
					// 該当マスを対象として含める
					positions.append(position)
					break
				}
			}
		}
		return positions
	}

	func 王を取る手(手番: Player) -> [Action] {
		var actions = [Action]()
		let 王の種類 = [Square.先玉, Square.後王] // [Square: [Position]]
		let dictionary = 駒からマスへの辞書()
		for 王 in 王の種類 {
			if let positions = dictionary[王], let 王の相手 = 王.先後 {
				for 王の位置 in positions {
					for 駒の位置 in 指定位置へ移動可能な全ての駒の位置(王の位置) {
						let square: Square = self[駒の位置]
						if let 先後 = 王.先後, 駒面 = self[駒の位置].駒面 {
							if 先後 == 王の相手 {
								let action = Action.動(先後: 王の相手, 移動前の位置: 駒の位置, 移動後の位置: 王の位置, 移動後の駒面: 駒面)
								actions.append(action)
							}
						}
					}
				}
			}
		}
		return actions
	}
	
	func 詰みか() -> Bool {
		// （未実装）
		return false
	}

	func positions() -> [Position] {
		// 玉の周りや、角、飛など優先順位に応じて、探索する順序を決めるベキ？
		return Position.positions
	}

	func generate() -> BoardGenerator {
		return BoardGenerator()
	}

/*
	var bitField : ZBitField {
		var spec: [ (String, Int) ] = [ ("row, 6"), ("row, 6"), ("row, 6"), ("row, 6"), ("row, 6") ]
		
		
		for row in 0...8 {
			let element = ("row\(index)", 6)
			spec.append(element)
		}
		return
	}
*/

	func data() -> NSData {
		var stream = NSOutputStream.outputStreamToMemory()
		var data = NSMutableData()
		for row in 0...8 {
			var value: UInt64 = 0
			for col in 0...8 {
				let square = self[col, row]
				value = value * UInt64(Square.max) + UInt64(square!.rawValue)
			}
			assert(value < 0x100_0000)
			for _ in 1...6 { // max 6 bytes
				var hex = UInt8(value % 0x100)
				data.appendBytes(&hex, length: 1)
				value /= 0x100
			}
		}
		var 先手持駒: UInt32 = CFSwapInt32HostToBig(captured[.先手]!.integerValue)
		var 後手持駒: UInt32 = CFSwapInt32HostToBig(captured[.後手]!.integerValue)
		data.appendBytes(&先手持駒, length: sizeof(UInt32))
		data.appendBytes(&後手持駒, length: sizeof(UInt32))
		return data
	}

	init(data: NSData) {
		var stream = NSInputStream(data: data)

		var squares = [Square](count: numberOfSquares, repeatedValue: Square.空)
		var captured = [Player.先手: Captured(), Player.後手: Captured()]
		if let 先手持駒 = stream.readUInt32(), 後手持駒 = stream.readUInt32() {
			
		}
		

		turn = .先手
		
		for row in 0...8 {
			
			var value: UInt64 = 0
			for _ in 1...6 { // max 6 bytes
				if let hex = stream.readUInt8() {
					value *= UInt64(0x100) ; value += UInt64(hex)
				}
			}

			for col in 0...8 {
				let squareValue = value % UInt64(Square.max) ; value /= UInt64(Square.max)
				if let index = Board.squareIndexFrom(column: col, row: row), square = Square(rawValue: Int8(squareValue)) {
					squares[index] = square
				}
			}


		}
		self.squares = squares
		self.captured = captured



	}
}

struct BoardGenerator: GeneratorType {
	private var rawIndex = Int8(0)
	mutating func next() -> Position? {
		if rawIndex < Int8(numberOfSquares) {
			return Position(rawValue: rawIndex++)
		}
		return nil
	}
}


func == (lhs: Board, rhs: Board) -> Bool {
	return true
}


class Game {
	var handicap: Handicap = .平手
	var actions = [Action]()
}

func + (lhs: (Int, Int), rhs: (Int, Int)) -> (Int, Int) {
	return (lhs.0 + rhs.0, lhs.1 + rhs.1)
}

func += (inout lhs: (Int, Int), rhs: (Int, Int)) {
	lhs = (lhs.0 + rhs.0, lhs.1 + rhs.1)
}

func ~= (lhs: (Range<Int>, Range<Int>), rhs: (Int, Int)) -> Bool {
	return lhs.0 ~= rhs.0 && lhs.1 ~= rhs.1
}

                                                                                                                                                                                                                                                                                                                  