//
//  NSInputStream+Z.swift
//  Kiban2
//
//  Created by Kaz Yoshikawa on 2015/05/23.
//  Copyright (c) 2015年 Electricwoods LLC. All rights reserved.
//

import Foundation


extension NSInputStream {

	func readUInt8() -> UInt8? {
		var result: UInt8? = nil
		var buffer = UnsafeMutablePointer<UInt8>.alloc(1)
		if self.read(UnsafeMutablePointer<UInt8>(buffer), maxLength: sizeof(UInt8)) == sizeof(UInt8) {
			result = buffer.memory
		}
		buffer.dealloc(1)
		return result
	}

	func readUInt16() -> UInt16? {
		var result: UInt16? = nil
		var buffer = UnsafeMutablePointer<UInt16>.alloc(1)
		if self.read(UnsafeMutablePointer<UInt8>(buffer), maxLength: sizeof(UInt16)) == sizeof(UInt16) {
			result = CFSwapInt16BigToHost(buffer.memory)
		}
		buffer.dealloc(1)
		return result
	}

	func readUInt32() -> UInt32? {
		var result: UInt32? = nil
		var buffer = UnsafeMutablePointer<UInt32>.alloc(1)
		if self.read(UnsafeMutablePointer<UInt8>(buffer), maxLength: sizeof(UInt32)) == sizeof(UInt32) {
			result = CFSwapInt32BigToHost(buffer.memory)
		}
		buffer.dealloc(1)
		return result
	}

	func readUInt64() -> UInt64? {
		var result: UInt64? = nil
		var buffer = UnsafeMutablePointer<UInt64>.alloc(1)
		if self.read(UnsafeMutablePointer<UInt8>(buffer), maxLength: sizeof(UInt64)) == sizeof(UInt64) {
			result = CFSwapInt64BigToHost(buffer.memory)
		}
		buffer.dealloc(1)
		return result
	}

	func readString() -> String? {
		var buffer = [UInt8]()
		while let byte = self.readUInt8() {
			buffer.append(byte)
			if byte == 0x00 { break }
		}
		return String(bytes: buffer, encoding: NSUTF8StringEncoding)
	}

}


extension NSOutputStream {

	func data() -> NSData? {
		return self.propertyForKey(NSStreamDataWrittenToMemoryStreamKey) as? NSData
	}

	func writeUInt8(value: UInt8) -> Bool {
		var buffer = UnsafeMutablePointer<UInt8>.alloc(1)
		buffer.memory = value
		let result = self.write(buffer, maxLength: sizeof(UInt8)) == sizeof(UInt8)
		buffer.dealloc(1)
		return result
	}

	func writeUInt16(value: UInt16) -> Bool {
		var buffer = UnsafeMutablePointer<UInt16>.alloc(1)
		buffer.memory = value
		let result = self.write(UnsafeMutablePointer<UInt8>(buffer), maxLength: sizeof(UInt16)) == sizeof(UInt16)
		buffer.dealloc(1)
		return result
	}

	func writeUInt32(value: UInt32) -> Bool {
		var buffer = UnsafeMutablePointer<UInt32>.alloc(1)
		buffer.memory = value
		let result = self.write(UnsafeMutablePointer<UInt8>(buffer), maxLength: sizeof(UInt32)) == sizeof(UInt32)
		buffer.dealloc(1)
		return result
	}

	func writeUInt64(value: UInt64) -> Bool {
		var buffer = UnsafeMutablePointer<UInt64>.alloc(1)
		buffer.memory = value
		let result = self.write(UnsafeMutablePointer<UInt8>(buffer), maxLength: sizeof(UInt64)) == sizeof(UInt64)
		buffer.dealloc(1)
		return result
	}

	func writeString(string: String) -> Bool {
		if let data = string.dataUsingEncoding(NSUTF8StringEncoding) {
			self.write(UnsafePointer<UInt8>(data.bytes), maxLength: data.length)
			self.writeUInt8(0x00) // null terminator
			return true
		}
		return false
	}

}